#!/bin/sh
# bb-ntp.sh
#
# BIG BROTHER - NTP EXTERNAL SCRIPT
# Bennett Samowich - Bennett Technologies
#
# This program is Copyright (c) 1997-1999
# Bennett Technologies
# All Rights Reserved

# version 1.0   Bennett Samowich
# version 1.1   Craig Cook            22 Oct 2001
#               Changed logic to look for "stratum" instead of "Timeout"
#               removed "-m 1" switch - did not work on Solaris
#               major style change, similar to bb-prtdiag.sh and
#                 other updated scripts
#               added functionality to check for the existence of "ntp"
#                 after the server name in bb-hosts, but now optional
#               changed &red and &yellow to variables
# version 1.2   Craig Cook            22 Nov 2001
#               added check for stratum 16 problems
#               corrected if then logic.  Now if ntptrace test fails it does
#                 not do further tests
#               added new comment line for testing, thanks to Andrew Clarke
# version 1.3	Craig Cook            09 Jan 2004
#               Changed version number to reflect deadcat numbering
# version 1.4	Craig Cook            09 Jan 2004
#               applied patch from Dan McDonald
#		- Shamelessly copied code from Matthew Lee Stier 
#		- To make it run on multiple hosts.
#               Changed patch to use an optional switch
#               Changed default panic time to 3 seconds
#                 (kerberos will fail logins at 5 secs)
# version 1.5	Craig Cook            26 Apr 2005
#               applied patch from Michel van de Visser	
#               - fixed incorect "test search", would also find a host
#                 with a nntp test
#               - fixed wrong $HOSTIP check
		
# Tested on Solaris and HPUX

# This script has two modes.

# 1. Execute on a local host, check the local time
# 2. Run from one host, but test multiple ntp devices.
#    - use this method when you can't run a local script, eg. cisco devices 
#    - to use this method each host to be tested must have the keyword "ntp"
#      on the host line eg.
#             myserver1.domain.com   # ntp
#             myserver2.domain.com   # ntp


###########################################################################
# INSTALLATION
#
#  
#  Option 1
#  Execute on a local host, check the local time
#
#  step 1  - update bb-bbexttab to include this bb-ntp.sh
#            (older BB versions update EXT section of the bbdef.sh script)
#
#  step 2 - copy lines mentioned to bbsys.local (without the #'s)
#
#  step 3 - if you are using an older version of BB without bb-bbexttab
#             and you don't want this run on every client uncomment
#             CHECK_BB_HOSTS="Y" and add the name of this $TEST to
#             bb-hosts for this client. eg.
#             myserver1.domain.com   # ntp
#
#  step 4 - restart Big Brother
#
#  Option 2
#  To run from one host, but test many devices
#
#  step 1 - update bb-bbexttab to include this bb-ntp.sh - only on the one
#            host executing this test
#            (older BB versions update EXT section of the bbdef.sh script)
#
#  step 2 - copy lines mentioned to bbsys.local (without the #'s)
#
#  step 3 - uncomment RUN_FROM_ONE_HOST=Y and add the name of this $TEST to
#             bb-hosts for each client. eg.
#             myserver1.domain.com   # ntp
#             myserver2.domain.com   # ntp
#
# NOTE - the TEST variable in the configuration section, this is the name
#        used as the column header and in the BB-HOSTS file.
###########################################################################

###########################################################################
# For testing, ensure BBHOME is set and uncomment the following line
#BB=""; BBDISP="echo"
###########################################################################

##################################
# CONFIGURE IT HERE
##################################
TEST="ntp"   # Column name this test will automatically show up under
BBPROG=$0; export BBPROG

#
# Start of lines to put in bbsys.local
#

#NTPTRACE="/usr/sbin/ntptrace"
#NTPTRACE_HOST="-n "

#export NTPTRACE NTPTRACE_OPT

#
# End of lines to put in bbsys.local
#

NTPWARN="2"	# GO YELLOW AT THIS LEVEL
NTPPANIC="3"	# GO RED AND PAGE AT THIS LEVEL

# define colours for graphics
RED_PIC="&red"
YELLOW_PIC="&yellow"

# don't scan through bb-hosts every time
# this is here for older BB versions without bb-bbexttab
# uncomment to activate
#CHECK_BB_HOSTS=Y

# Use this option to execute from one host, but check many ntp devices
# uncomment to activate
#RUN_FROM_ONE_HOST=Y


##################################
# Start of script
##################################

if test ! "$BBHOME"
then
        echo "template: BBHOME is not set"
        exit 1
fi

if test ! -d "$BBHOME"
then
        echo "template: BBHOME is invalid"
        exit 1
fi

if test ! "$BBTMP"                      # GET DEFINITIONS IF NEEDED
then
        echo "template: The BB environment is not set"
        . $BBHOME/etc/bbdef.sh          # INCLUDE STANDARD DEFINITIONS
fi

get_header()
{
  echo ""
  echo "<FONT SIZE=+2><b>$1</b></FONT> ($2 $3)<BR>"
  #If you do not want the header in a bigger font use line below instead
  #  echo "<b>$1</b> ($2 $3)"
}

get_footer()
{
  echo ""
  echo ""
}

#####
#####  Get Status proc - used to get all responses
#####
get_status()
{
  # Define default color
  COLOR="green"

  # Check defaults have been set
  if [ "$NTPTRACE" = "" ]
  then
    NTPTRACE="/usr/sbin/ntptrace"
    echo ""
    echo "$YELLOW_PIC NTPTRACE command is not defined in etc/bbsys.local - using default: $NTPTRACE"
  fi

  #
  # TEST IF NTP IS RUNNING ON THE REMOTE MACHINE BY USING
  # NTPTRACE TO QUERY THE TIME DIFFERENCE.  WE WILL CREATE
  # ALERTS BASED ON THE AMOUNT OF DRIFT.
  #
  get_header "NTP Info" "$NTPTRACE" "$NTPTRACE_OPT"

  CATCH_COMMAND=` $NTPTRACE $NTPTRACE_OPT `

  if [ "$?" != "0" ] ; then
      COLOR="red"
      echo "$RED_PIC Problem running $NTPTRACE $NTPTRACE_OPT !!!"
  else

    echo "$CATCH_COMMAND" | $GREP -i "stratum" >/dev/null 2>&1

    if [ $? != 0 ] ; then
      COLOR="red"
      UD="down"
    else
      # NTP must be running.  If stratum 16 is found, it is not happy
      echo "$CATCH_COMMAND" | $GREP -i "stratum 16" >/dev/null 2>&1
      if [ $? = 0 ] ; then
        echo "$YELLOW_PIC ntpd is running, but not synchronised!!!"
        COLOR="yellow"
        UD="up"
        MARK="PROBLEM"
      else
        UD="up"
        OFFSET=`echo $CATCH_COMMAND | $AWK '{print $5}' | \
                         $SED 's/,//'`
        OFFSET_WHOLE_SECONDS=`echo $OFFSET | $SED 's/-//' |   \
                         $SED 's/\./ /' | \
                         $AWK '{print $1}'`
        MARK="OK"
        if [ "$OFFSET_WHOLE_SECONDS" -ge "$NTPPANIC" ] ; then
              COLOR="red"
              MARK="$RED_PIC PANIC"
        elif [ "$OFFSET_WHOLE_SECONDS" -ge "$NTPWARN" ] ; then
              COLOR="yellow"
              MARK="$YELLOW_PIC WARNING"
        fi

      fi
    fi

    echo "NTP server is $UD 
$MARK: Clock offset is ($OFFSET)

$CATCH_COMMAND"

  fi

  get_footer

}

#####
#####  Main body
#####
if [ "$CHECK_BB_HOSTS" = "Y" ]; then
  # convert "," to "." in the hostname
  MACHINE_WITH_DOTS=`echo $MACHINE | $SED 's/,/\./g'`

  $GREP $MACHINE_WITH_DOTS $BBHOSTS | $GREP "$TEST" |
  while read line
  do
    if [ ! -z "$line" ]; then
      get_status > $BBTMP/$MACHINE.$TEST

      # NOW USE THE BB COMMAND TO SEND THE DATA ACROSS
      $BB $BBDISP "status $BBTMP/$MACHINE.$TEST $COLOR `$DATE` `$CAT $BBTMP/$MACHINE.$TEST` "
    fi
  done

elif [ "$RUN_FROM_ONE_HOST" = "Y" ]; then

  #$GREP -e ^[0-9] $BBHOSTS | $GREP "$TEST" |
  $EGREP -e \^\[0-9\] $BBHOSTS | $SED 's/nntp//' | $GREP $TEST |
  while read line
  do
    set $line
    HOSTIP=${1}
    if [ "$HOSTIP" = "0.0.0.0" ]; then
	    HOSTIP=${2}
    fi
    MACHINETOTEST="`echo ${2} | $SED 's/\./,/g'`";
    NTPTRACE_OPT=$NTPTRACE_HOST$HOSTIP
	  
    if [ ! -z "$line" ]; then
      get_status > $BBTMP/$MACHINETOTEST.$TEST

      # NOW USE THE BB COMMAND TO SEND THE DATA ACROSS
      $BB $BBDISP "status $MACHINETOTEST.$TEST $COLOR `$DATE` `$CAT $BBTMP/$MACHINETOTEST.$TEST` "
    fi
  done
else
  NTPTRACE_OPT=$NTPTRACE_HOST
  get_status > $BBTMP/$MACHINE.$TEST

  # NOW USE THE BB COMMAND TO SEND THE DATA ACROSS
  $BB $BBDISP "status $MACHINE.$TEST $COLOR `$DATE` `$CAT $BBTMP/$MACHINE.$TEST`
 "
fi

# Clean up our mess
# Checking for existence of each file since the whole test may be optional
#   and may not actually run on every client
#
if [ -f $BBTMP/$MACHINE.$TEST ]; then
  $RM $BBTMP/$MACHINE.$TEST
fi
if [ -f $BBTMP/$MACHINETOTEST.$TEST ]; then
  $RM $BBTMP/$MACHINETOTEST.$TEST
fi
